#!/bin/bash
#
# Copyright (c) 2014, Intel Corporation
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#    * Neither the name of Intel Corporation nor the names of its contributors
#      may be used to endorse or promote products derived from this software
#      without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#

echo "Before running this script, you must edit file 'onps_config'"
echo -e "and enter system specific information \n "

read -p "If you are ready, press 'y' or 'Y' to continue " response
case $response in
    [yY]* ) echo -e "start preparing fedora system \n" ;;
    * ) echo -e "exit to edit onps_config file \n "; exit;;
esac

# import necessary system information
source onps_config


# perform the following as 'su' user
# ======================================

# rename network interface for easier use in the script
net1=$mgmt_net_if
net2=$inter_net_if 
net3=$virtual_net_if 
net4=$public_net_if 

# other infomation
file_path=$PWD

# set host name by editing /etc/hostname
echo -e "set host name \n"
existing_name=`cat /etc/hostname`
sed -i 's/'$existing_name'/'$host_name'/g' /etc/hostname
sleep 1
echo -e "done change host name \n"

# configure networking, edit /etc/sysconfig/network-scripts/ifcfg-net-interface
# four netwrok interfaces to be configured 
#	net1 - used for openstack managerment
#	net2 - used for internet network, pulling packages from all repositories
#	net3 - used for openstack virtual network (tenant network)
#	net4 - used for openstack public network for external connections
echo -e "configure network interfaces \n"
# net1
onboot=`grep ONBOOT /etc/sysconfig/network-scripts/ifcfg-$net1`
proto=`grep BOOTPROTO /etc/sysconfig/network-scripts/ifcfg-$net1`
sed -i 's/'$onboot'/ONBOOT=yes/' /etc/sysconfig/network-scripts/ifcfg-$net1
sed -i 's/'$proto'/BOOTPROTO=static/' /etc/sysconfig/network-scripts/ifcfg-$net1
sed -i '/BOOTPROTO=static/a \IPADDR='$ip_addr' \nNETMASK='$net_mask'' /etc/sysconfig/network-scripts/ifcfg-$net1
sed -i 's/DEFROUTE="yes"/DEFROUTE=no/' /etc/sysconfig/network-scripts/ifcfg-$net1
sleep 1
echo -e "done configure $net1 \n"

# net2
onboot=`grep ONBOOT /etc/sysconfig/network-scripts/ifcfg-$net2`
sed -i 's/'$onboot'/ONBOOT=yes/' /etc/sysconfig/network-scripts/ifcfg-$net2
sleep 1
echo -e "done configure $net2 \n"

# net3 
onboot=`grep ONBOOT /etc/sysconfig/network-scripts/ifcfg-$net3`
proto=`grep BOOTPROTO /etc/sysconfig/network-scripts/ifcfg-$net3`
sed -i 's/'$onboot'/ONBOOT=yes/' /etc/sysconfig/network-scripts/ifcfg-$net3
sed -i 's/'$proto'/BOOTPROTO=none/' /etc/sysconfig/network-scripts/ifcfg-$net3
sleep 1
echo -e "done configure $net3 \n"

# net4
onboot=`grep ONBOOT /etc/sysconfig/network-scripts/ifcfg-$net4`
proto=`grep BOOTPROTO /etc/sysconfig/network-scripts/ifcfg-$net4`
sed -i 's/'$onboot'/ONBOOT=yes/' /etc/sysconfig/network-scripts/ifcfg-$net4
sed -i 's/'$proto'/BOOTPROTO=none/' /etc/sysconfig/network-scripts/ifcfg-$net4
sleep 1
echo -e "done configure $net4 \n"

# restart network
systemctl restart NetworkManager.service
sleep 1

# looks like need to shutdown net1, otherwise it will take default gateway away and cause no access to internet for remaining work 
ifdown $net1
sleep 5
echo -e "done network config \n"

# configure proxy for http, https and ftp 
echo -e "configure proxy settings \n"
echo "export http_proxy=$ONPS_HTTP_PROXY:$ONPS_HTTP_PROXY_PORT" >> /root/.bashrc
echo "export https_proxy=$ONPS_HTTPS_PROXY:$ONPS_HTTPS_PROXY_PORT" >> /root/.bashrc
echo "export ftp_proxy=$ONPS_FTP_PROXY:$ONPS_FTP_PROXY_PORT" >> /root/.bashrc
echo "export no_proxy=$NO_PROXY" >> /root/.bashrc
sleep 1

export http_proxy=http://$ONPS_HTTP_PROXY:$ONPS_HTTP_PROXY_PORT
export https_proxy=https://$ONPS_HTTPS_PROXY:$ONPS_HTTPS_PROXY_PORT
export ftp_proxy=$ONPS_FTP_PROXY:$ONPS_FTP_PROXY_PORT
export no_proxy=$NO_PROXY 
sleep 1
echo -e "done http, https, and ftp proxy configurations \n"

# configure yum proxy, editing /etc/yum.conf
# limit to current kernel version
echo -e "configure settings \n"
echo "proxy=http://$ONPS_HTTP_PROXY:$ONPS_HTTP_PROXY_PORT" >> /etc/yum.conf

# if choose using specific kernel version, need to do this 
if [ $use_latest_kernel == 0 ]
then
	echo "exclude=kernel*" >> /etc/yum.conf
fi
sleep 2
echo -e "done yum configuration\n"

# configure git 
# install git package
echo -e "configure git  \n"
yum install -y git
sleep 1
echo -e "done installing git packages \n"

# create gitconnect in /usr/local/bin
echo 'exec socat STDIO SOCKS4:onps_http_proxy:$1:$2' > /usr/local/bin/gitconnect
sed -i "s/SOCKS4:onps_http_proxy/SOCKS4:$ONPS_HTTP_PROXY/" /usr/local/bin/gitconnect
chmod +x /usr/local/bin/gitconnect

# set git proxy, create and edit .gitconfig
echo "[http]" > /root/.gitconfig
echo "    proxy = $ONPS_HTTP_PROXY:$ONPS_HTTP_PROXY_PORT" >> /root/.gitconfig
echo "[https]" >> /root/.gitconfig
echo "    proxy = $ONPS_HTTPS_PROXY:$ONPS_HTTPS_PROXY_PORT" >> /root/.gitconfig
sleep 1

echo "[core]" >> /root/.gitconfig
echo "    gitproxy = none for $NO_GIT_PROXY" >> /root/.gitconfig
echo "    gitproxy = /usr/local/bin/gitconnect" >> /root/.gitconfig
sleep 1
echo -e "done git configuration \n"

# configure ntp
echo -e "configure NTP  \n"
yum install -y ntp
sleep 1
echo -e "done installing ntp packages \n"

# configure ntp server
echo "server $ntp_server_name" >> /etc/ntp.conf
sed -i 's/server 0.fedora.pool.ntp.org iburst/#server 0.fedora.pool.ntp.org iburst/g' /etc/ntp.conf
sed -i 's/server 1.fedora.pool.ntp.org iburst/#server 1.fedora.pool.ntp.org iburst/g' /etc/ntp.conf
sed -i 's/server 2.fedora.pool.ntp.org iburst/#server 2.fedora.pool.ntp.org iburst/g' /etc/ntp.conf
sed -i 's/server 3.fedora.pool.ntp.org iburst/#server 3.fedora.pool.ntp.org iburst/g' /etc/ntp.conf
sleep 1
echo "done NTP configuration \n"

# install necessary packages
echo "install necessary packages \n"
# packages need for SRT standard OS installation
yum install -y patch socat python-passlib libxslt-devel libffi-devel fuse-devel gluster python-cliff 
sleep 1

# user preference packages here (not required)
yum install -y vim-minimal
yum install -y vim
sleep 1
echo -e "done yum package installation \n"


# install pip and python components
wget https://pypi.python.org/packages/source/p/pip/pip-1.4.1.tar.gz
tar zxvf pip-1.4.1.tar.gz

cd pip-1.4.1/
sudo -E python setup.py install
echo -e "done install pip 1.4.1 \n"
sleep 5

pip install dogpile.cache 
echo -e "done install dogpile \n"
sleep 5

upgrade requests to newer version 
pip install --upgrade requests==2.4.0
echo -e "done upgrade requests \n"
sleep 5
cd ../


# disable/enable services for openstack 
echo -e "enable/disable services for openstack needs \n" 
# disable selinux
sed -i 's/SELINUX=enforcing/SELINUX=disabled/' /etc/selinux/config
sleep 1

# disable firewalld
systemctl stop firewalld.service
systemctl disable firewalld.service
sleep 1

# disable Networkmanager
systemctl stop NetworkManager.service
systemctl disable NetworkManager.service
sleep 1

# enable network service
#service network start
chkconfig network on
sleep 1

# enable ntpd
systemctl start ntpd.service
systemctl enable ntpd.service
sleep 1

# enable sshd
systemctl start sshd.service
systemctl enable sshd.service
sleep 1
echo -e "done service configuration \n"

# update timing against ntp server
ntpdate -u $ntp_server_name
sleep 1
echo -e "done sync timing \n"


# edit /etc/libvirt/qemu.conf for DPDK if this is a compute node
# Note: huge page mount point is /mnt/huge
shopt -s nocasematch
if [ "$node_type" == "compute" ]
then
	echo -e "configure libvirt for DPDK needs \n "
	echo 'cgroup_controllers = [ "cpu", "devices", "memory", "blkio", "cpuset", "cpuacct" ]' >> /etc/libvirt/qemu.conf
	echo 'hugetlbfs_mount = "/mnt/huge"' >> /etc/libvirt/qemu.conf
	echo 'cgroup_device_acl = [' >> /etc/libvirt/qemu.conf
	echo '    "/dev/null", "/dev/full", "/dev/zero",' >> /etc/libvirt/qemu.conf
	echo '    "/dev/random", "/dev/urandom",' >> /etc/libvirt/qemu.conf
	echo '    "/dev/ptmx", "/dev/kvm", "/dev/kqemu",' >> /etc/libvirt/qemu.conf
	echo '    "/dev/rtc", "/dev/hpet", "/dev/net/tun",' >> /etc/libvirt/qemu.conf
	echo '     "/mnt/huge","/dev/vhost-net"]' >> /etc/libvirt/qemu.conf

	systemctl restart libvirtd.service
	echo -e "done updating /etc/libvirt/qeum.conf \n"
	sleep 1
fi
shopt -u nocasematch


# if choose using specific kernel, need to download and install rpm packages 
# otherwise run yum update to get the latest kernel 
if [ $use_latest_kernel == 0 ]
then  
	# download kernel $version  packages and install
	echo -e "download kernel $version  \n"
	wget $kernelURL/kernel-$version.fc20.x86_64.rpm
	wget $kernelURL/kernel-devel-$version.fc20.x86_64.rpm
	wget $kernelURL/kernel-modules-extra-$version.fc20.x86_64.rpm
	sleep 1
	echo -e "done kernel $version downloads  \n"

	# install $version kernel
	echo -e "installing  kernel $version  \n"
	rpm -i kernel-$version.fc20.x86_64.rpm
	rpm -i kernel-devel-$version.fc20.x86_64.rpm
	rpm -i kernel-modules-extra-$version.fc20.x86_64.rpm
	sleep 1
	echo -e "done kernel $version install  \n"
fi


# Now working with user 'stack'
#    Note: 'stack' is already created during OS installation
# ======================================
echo "stack user configurations \n" 
# add 'stack' to sudoer
echo "stack ALL=(ALL) NOPASSWD: ALL" >> /etc/sudoers
sleep 1
echo -e "done adding sudoer for stack user \n"

# find internet network ip address
inet_ip=`ifconfig $net2 | grep netmask | awk '{print $2}'`
sleep 1 

# set stack user proxy, edit /home/stack/.bashrc and apply to env
echo "export http_proxy=http://$ONPS_HTTP_PROXY:$ONPS_HTTP_PROXY_PORT" >> /home/stack/.bashrc
echo "export https_proxy=https://$ONPS_HTTPS_PROXY:$ONPS_HTTPS_PROXY_PORT" >> /home/stack/.bashrc
echo "export ftp_proxy=ftp://$ONPS_FTP_PROXY:$ONPS_FTP_PROXY_PORT" >> /home/stack/.bashrc
echo "export no_proxy=localhost,$host_name,$ip_addr,$inet_ip,$controller_name,$controller_ip,$NO_PROXY" >> /home/stack/.bashrc
sleep 2
echo -e "done proxy for stack user \n"

# set stack user git proxy
echo "[http]" > /home/stack/.gitconfig
echo "    proxy = $ONPS_HTTP_PROXY:$ONPS_HTTP_PROXY_PORT" >> /home/stack/.gitconfig

echo "[https]" >> /home/stack/.gitconfig
echo "    proxy = $ONPS_HTTPS_PROXY:$ONPS_HTTPS_PROXY_PORT" >> /home/stack/.gitconfig

echo "[core]" >> /home/stack/.gitconfig
echo "    gitproxy = none for $NO_GIT_PROXY" >> /home/stack/.gitconfig
echo "    gitproxy = /usr/local/bin/gitconnect" >> /home/stack/.gitconfig
sleep 1

chown stack:stack /home/stack/.gitconfig
sleep 1
echo -e "done git proxy for stack user \n"


# copy prepare_stack file for stack user
cp $file_path/prepare_stack.sh /home/stack/
chown stack:stack /home/stack/prepare_stack.sh
sleep 1
echo -e "done prepare_stack script \n"


# download and copy openstack-ovdk patches to stack directory
mkdir /home/stack/patches
chown stack:stack /home/stack/patches
sleep 1

wget $openstack_patch_URL
sleep 1
#tar zxvf $openstack_patch_name --directory=/home/stack/patches/
unzip $openstack_patch_name -d /home/stack/patches/
chown stack:stack /home/stack/patches/*
sleep 1
echo -e "done copy patch files to stack directory \n"


# create local.conf file 
shopt -s nocasematch
if [ $node_type == "controller" ] 
then
    cp $file_path/local.conf-ovdk-controller /home/stack/local.conf
    sleep 1

    sed -i 's/HOST_IP=/HOST_IP='$ip_addr'/g' /home/stack/local.conf
    sed -i 's/HOST_IP_IFACE=/HOST_IP_IFACE='$net1'/g' /home/stack/local.conf
    sed -i 's/PUBLIC_INTERFACE=/PUBLIC_INTERFACE='$net4'/g' /home/stack/local.conf
    sed -i 's/VLAN_INTERFACE=/VLAN_INTERFACE='$net3'/g' /home/stack/local.conf
    sed -i 's/FLAT_INTERFACE=/FLAT_INTERFACE='$net3'/g' /home/stack/local.conf
    sed -i 's/OVS_PHYSICAL_BRIDGE=/OVS_PHYSICAL_BRIDGE=br-'$net3'/g' /home/stack/local.conf    
else
    cp $file_path/local.conf-ovdk-compute /home/stack/local.conf
    sleep 1

    sed -i 's/HOST_IP=/HOST_IP='$ip_addr'/g' /home/stack/local.conf
    sed -i 's/vncserver_proxyclient_address=/vncserver_proxyclient_address='$ip_addr'/g' /home/stack/local.conf
    sed -i 's/HOST_IP_IFACE=/HOST_IP_IFACE='$net1'/g' /home/stack/local.conf
    sed -i 's/SERVICE_HOST_NAME=/SERVICE_HOST_NAME='$controller_ip'/g' /home/stack/local.conf
    sed -i 's/SERVICE_HOST=/SERVICE_HOST='$controller_ip'/g' /home/stack/local.conf
    sed -i 's/OVS_PHYSICAL_BRIDGE=/OVS_PHYSICAL_BRIDGE=br-'$net3'/g' /home/stack/local.conf

	if [ $q_agent == "ovdk" ] 
	then
	    sed -i 's/Q_AGENT=/Q_AGENT=ovdk/g' /home/stack/local.conf
            sed -i 's/OVDK_OVS_GIT_TAG=/OVDK_GIT_TAG='$ovdk_git_tag'/g' /home/stack/local.conf
            sed -i '/Q_ML2_PLUGIN_TYPE_DRIVERS/a \OVDK_NUM_HUGEPAGES=8192' /home/stack/local.conf

	else
	    sed -i 's/Q_AGENT=/Q_AGENT=openvswitch/g' /home/stack/local.conf
            sed -i 's/OVDK_OVS_GIT_TAG=/OVS_GIT_TAG='$ovs_git_tag'/g' /home/stack/local.conf
            sed -i '/Q_ML2_PLUGIN_TYPE_DRIVERS/a \OVS_NUM_HUGEPAGES=8192 \nOVS_DATAPATH_TYPE=netdev' /home/stack/local.conf
	fi
fi
sleep 1
shopt -u nocasematch


chown stack:stack /home/stack/local.conf
sleep 1
echo -e "done creating local.conf file \n"


# done system preparations
echo -e "done fedora system preparations \n"
echo -e "next steps: \n"
echo -e "    1. reboot the system \n"
echo -e "    2. login as root, run command 'yum update -y',  then reboot again \n"
echo -e "    3. login as stack \n"
echo -e "    4. run script 'prepare_stack.sh' to install openstack\n"
echo

exit 1

